import type { FeatureRegistry } from '../../agent/FeatureRegistry';
import type { ApiModule, DependencyManager } from '../../plugins';
import type { Constructor } from '../../utils/mixins';
import type { Optional } from '../../utils/type';
import type { CredentialsModuleConfigOptions } from './CredentialsModuleConfig';
import type { CredentialProtocol } from './protocol/CredentialProtocol';
import { CredentialsApi } from './CredentialsApi';
import { CredentialsModuleConfig } from './CredentialsModuleConfig';
import { IndyCredentialFormatService } from './formats/indy';
import { V1CredentialProtocol } from './protocol/v1';
import { V2CredentialProtocol } from './protocol/v2';
/**
 * Default credentialProtocols that will be registered if the `credentialProtocols` property is not configured.
 */
export declare type DefaultCredentialProtocols = [V1CredentialProtocol, V2CredentialProtocol<IndyCredentialFormatService[]>];
export declare type CredentialsModuleOptions<CredentialProtocols extends CredentialProtocol[]> = Optional<CredentialsModuleConfigOptions<CredentialProtocols>, 'credentialProtocols'>;
export declare class CredentialsModule<CredentialProtocols extends CredentialProtocol[] = DefaultCredentialProtocols> implements ApiModule {
    readonly config: CredentialsModuleConfig<CredentialProtocols>;
    readonly api: Constructor<CredentialsApi<CredentialProtocols>>;
    constructor(config?: CredentialsModuleOptions<CredentialProtocols>);
    /**
     * Get the default credential protocols that will be registered if the `credentialProtocols` property is not configured.
     */
    private getDefaultCredentialProtocols;
    /**
     * Registers the dependencies of the credentials module on the dependency manager.
     */
    register(dependencyManager: DependencyManager, featureRegistry: FeatureRegistry): void;
}
